import logging
import formencode
import formencode.htmlfill

from pylons import request
from pylons.controllers.util import redirect

from repoze.what.plugins.pylonshq import ControllerProtector

from webhelpers.paginate import Page

from sloecode.lib.base import BaseController
from sloecode.model.project import Project, ProjectSchema
from sloecode.model.meta import Session
from sloecode.lib.predicates import is_site_admin
import sloecode.lib.helpers as h

log = logging.getLogger(__name__)


@ControllerProtector(is_site_admin())
class ProjectController(BaseController):
    """ProjectController contains basic admin functionality for the Project
    model.

    This includes basic CRUD information.
    """

    def index(self):
        "Render basic project administration page."

        return self.render('/project-manage.html')

    def create(self):
        "Render project creation form."

        return self.render('/project-create.html')

    def process_create(self):
        "Process project creation request, redirect to project list view."

        schema = ProjectSchema()
        try:
            form_result = schema.to_python(request.params)
        except formencode.Invalid, e:
            errors = e.unpack_errors()
            return formencode.htmlfill.render(self.create(), defaults=request.params, errors=errors)

        new_project = Project()
        new_project.name = form_result['name']
        new_project.description = form_result['description']

        Session.add(new_project)
        Session.commit()
        h.flash("Project Added Successfully")
        return redirect('/project/retrieve')

    def retrieve(self):
        "Render a list of all projects on the system."
        curr_page = int(request.params.get('page', 1))
        page = Page(Project.get(), page=curr_page)

        return self.render('/project-list.html', {'projects': page})

    def update(self, id):
        "Render project update form page."

        return self.render('/project-update.html', {'project': Project.get(id=id)[0]})

    def process_update(self, id):
        "Process project update request."

        schema = ProjectSchema()
        del schema.fields['name']
        try:
            form_result = schema.to_python(request.params)
        except formencode.Invalid, e:
            errors = e.unpack_errors()
            return formencode.htmlfill.render(self.update(id), defaults=request.params, errors=errors)

        project = Project.get(id=id)[0]
        project.description = form_result['description']

        Session.merge(project)
        Session.commit()

        h.flash("Project Updated Successfully")
        return redirect('/project/retrieve')

    def delete(self, id):
        "Process project delete request."

        project = Project.get(id=id)[0]
        Session.delete(project)
        Session.commit()
        h.flash("Project Deleted Successfully")
        return redirect('/project/retrieve')
